@@ -278,6 +278,16 @@ static const canon_inkset_t canon_6_C7M7Y7K7c7m7_inkset[] = {
         {'m',1.0,&canon_3b_7l_ink}
 };
 
+static const canon_inkset_t canon_7_C4M4Y4c4m4k4K4_inkset[] = {
+        {'C',1.0,&canon_2b_4l_ink},
+        {'M',1.0,&canon_2b_4l_ink},
+        {'Y',1.0,&canon_2b_4l_ink},
+        {'c',1.0,&canon_2b_4l_ink},
+        {'m',1.0,&canon_2b_4l_ink},
+        {'k',1.0,&canon_2b_4l_ink},
+        {'K',1.0,&canon_2b_4l_ink},
+};
+
 static const canon_inkset_t canon_9_C3M3Y2K2_inkset[] = {
         {'C',1.0,&canon_2b_3l_ink},
         {'M',1.0,&canon_2b_3l_ink},
