@@ -1,5 +1,5 @@
 /*
- * "$Id: dither-main.c,v 1.52.8.1 2007/12/15 20:35:44 rlk Exp $"
+ * "$Id: dither-main.c,v 1.62 2010/08/04 00:33:56 rlk Exp $"
  *
  *   Dither routine entrypoints
  *
@@ -62,10 +62,13 @@ static const stpi_dither_algorithm_t dither_algos[] =
 #endif
   { "Adaptive",	      N_ ("Adaptive Hybrid"),        D_ADAPTIVE_HYBRID },
   { "Ordered",	      N_ ("Ordered"),                D_ORDERED },
+  { "OrderedNew",     N_ ("Ordered New"),            D_ORDERED_NEW },
   { "Fast",	      N_ ("Fast"),                   D_FAST },
   { "VeryFast",	      N_ ("Very Fast"),              D_VERY_FAST },
   { "Floyd",	      N_ ("Hybrid Floyd-Steinberg"), D_FLOYD_HYBRID },
-  { "Predithered",    N_ ("Predithered Input"),      D_PREDITHERED }
+  { "Predithered",    N_ ("Predithered Input"),      D_PREDITHERED },
+  { "Segmented",      N_ ("Drop Size Segmented"),    D_ORDERED_SEGMENTED },
+  { "SegmentedNew",   N_ ("Drop Size Segmented New"),D_ORDERED_SEGMENTED_NEW }
 };
 
 static const int num_dither_algos = sizeof(dither_algos)/sizeof(stpi_dither_algorithm_t);
@@ -91,7 +94,7 @@ static const stp_parameter_t dither_parameters[] =
        "paper or smears; increase the density if black "
        "regions are not solid."),
     STP_PARAMETER_TYPE_DOUBLE, STP_PARAMETER_CLASS_OUTPUT,
-    STP_PARAMETER_LEVEL_ADVANCED, 0, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_ADVANCED, 0, 1, STP_CHANNEL_NONE, 1, 0
   },
   {
     "DitherAlgorithm", N_("Dither Algorithm"), N_("Screening Adjustment"),
@@ -102,7 +105,7 @@ static const stp_parameter_t dither_parameters[] =
        "Fast and Very Fast are considerably faster, and work well for text and line art.\n"
        "Hybrid Floyd-Steinberg generally produces inferior output."),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_OUTPUT,
-    STP_PARAMETER_LEVEL_ADVANCED, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_ADVANCED, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
 };
 
@@ -175,6 +178,9 @@ stpi_set_dither_function(stp_vars_t *v)
 
   if (color_correction && strcmp(color_correction, "Predithered") == 0)
     d->stpi_dither_type = D_PREDITHERED;
+  else if (color_correction && strcmp(color_correction, "Raw") != 0 &&
+	   algorithm && strcmp(algorithm, "Segmented") == 0)
+    d->stpi_dither_type = D_ORDERED;
   else if (algorithm && strcmp(algorithm, "None") != 0)
     {
       for (i = 0; i < num_dither_algos; i++)
@@ -248,6 +254,8 @@ stpi_set_dither_function(stp_vars_t *v)
 	  (d->x_aspect > 2 || d->y_aspect > 2))
 	d->stpi_dither_type = D_ADAPTIVE_HYBRID;
     }
+  if (d->stpi_dither_type == -1)
+    d->stpi_dither_type = D_ADAPTIVE_HYBRID;
   switch (d->stpi_dither_type)
     {
     case D_PREDITHERED:
@@ -255,6 +263,9 @@ stpi_set_dither_function(stp_vars_t *v)
     case D_VERY_FAST:
       RETURN_DITHERFUNC(stpi_dither_very_fast, v);
     case D_ORDERED:
+    case D_ORDERED_SEGMENTED:
+    case D_ORDERED_NEW:
+    case D_ORDERED_SEGMENTED_NEW:
     case D_FAST:
       RETURN_DITHERFUNC(stpi_dither_ordered, v);
     case D_HYBRID_EVENTONE:
@@ -323,7 +334,6 @@ stpi_dither_free(void *vd)
   STP_SAFE_FREE(d->offset0_table);
   STP_SAFE_FREE(d->offset1_table);
   stp_dither_matrix_destroy(&(d->dither_matrix));
-  stp_dither_matrix_destroy(&(d->transition_matrix));
   stp_free(d->channel);
   stp_free(d->channel_index);
   stp_free(d->subchannel_count);
@@ -356,7 +366,6 @@ stp_dither_init(stp_vars_t *v, stp_image_t *image, int out_width,
       d->y_aspect = 1;
     }
   d->ditherfunc = stpi_set_dither_function(v);
-  d->transition = 1.0;
   d->adaptive_limit = .75 * 65535;
 
   /*
@@ -384,22 +393,13 @@ stp_dither_init(stp_vars_t *v, stp_image_t *image, int out_width,
     }
   else
     {
-      stp_array_t *array;
-      int transposed;
-	array = stp_find_standard_dither_array(d->y_aspect, d->x_aspect);
-      transposed = d->y_aspect < d->x_aspect ? 1 : 0;
-      if (array)
-	{
-	  stp_dither_set_matrix_from_dither_array(v, array, transposed);
-	  stp_array_destroy(array);
-	}
-      else
-	{
-	  stp_eprintf(v, "Cannot find dither matrix file!  Aborting.\n");
-	  stp_abort();
-	}
+      stp_array_t *array =
+	stp_find_standard_dither_array(d->y_aspect, d->x_aspect);
+      int transposed = d->y_aspect < d->x_aspect ? 1 : 0;
+      STPI_ASSERT(array, v);
+      stp_dither_set_matrix_from_dither_array(v, array, transposed);
+      stp_array_destroy(array);
     }
-  stp_dither_set_transition(v, 0.7);
 
   d->src_width = in_width;
   d->dst_width = out_width;
@@ -467,7 +467,6 @@ stp_dither_internal(stp_vars_t *v, int row, const unsigned short *input,
   stpi_dither_t *d = (stpi_dither_t *) stp_get_component_data(v, "Dither");
   stpi_dither_finalize(v);
   stp_dither_matrix_set_row(&(d->dither_matrix), row);
-  stp_dither_matrix_set_row(&(d->transition_matrix), row);
   for (i = 0; i < CHANNEL_COUNT(d); i++)
     {
       CHANNEL(d, i).ptr = CHANNEL(d, i).ptr;
